SUPPORTED_TYPES = [
    'int',
    'float',
    'bool',
    'str',
    'list',
    'dict',
    'tuple',
    'set',
    'datetime.date',
    'datetime.datetime',
    'datetime.time',
    'datetime.timedelta',
    'typing.List',
    'typing.Set',
    'typing.Dict',
    'typing.Tuple',
    'typing.Union',
    'typing.Optional',
    'typing.Any',
]
